class ChatController < ApplicationController
  layout 'chat', :only => :index

  def index
    if session[:theme].blank? || session[:theme].empty?
      @chats = Chat.find(:all)
    else
      @chats = Chat.find_tagged_with(session[:theme].join(','))
    end
  end

  def show
    @chat = Chat.find(params[:id])
  end

  def listen
    session[:name] = params[:name]
    session[:theme] = params[:theme].split(/\s+/)
    render :update do |page|
      page << <<-"EOH"
        meteorStrike['chat'].update(
          #{session[:name].to_json},
          #{session[:theme].to_json});
      EOH
    end
  end

  def talk
    @chat = Chat.create(:name => session[:name], :message => params[:message])
    @chat.tag_list = session[:theme].join(',')
    if @chat.save
      content = render_component_as_string :action => 'show', :id => @chat.id
      javascript = render_to_string :update do |page|
        page.insert_html :bottom, 'chat-list', content
      end
      Meteor.shoot 'chat', javascript, session[:theme]
      render :update do |page|
        page[:message].clear
        page[:message].focus
      end
    else
      render :nothing => true
    end
  end

  def event
    message = case params[:event]
    when 'init'; "connection established by #{params[:type]}."
    when 'enter'; "#{params[:uid]} joined."
    when 'leave'; "#{params[:uid]} left."
    end
    @chat = Chat.new(:name => '(* system *)', :created_at => Time.now,
      :message => message)
    render :action => 'show'
  end
end
